@@ -49,6 +49,7 @@ activate <- function(project = NULL, profile = NULL) {
   project <- renv_project_resolve(project)
   renv_project_lock(project = project)
 
+  profile <- profile %||% renv_profile_get()
   renv_profile_set(profile)
 
   renv_activate_impl(
@@ -164,7 +165,7 @@ renv_activate_prompt <- function(action, library, prompt, project) {
     interactive() &&
     is.null(library) &&
     !renv_project_loaded(project) &&
-    !is_testing()
+    !testing()
 
   # for snapshot, since users might want to snapshot their system library
   # in an renv-lite configuration, only prompt if it looks like they're
