@@ -1,6 +1,7 @@
 
 # aliases used primarily for nicer / normalized text output
-`_renv_aliases` <- list(
+the$aliases <- list(
+  bioc         = "Bioconductor",
   bioconductor = "Bioconductor",
   bitbucket    = "Bitbucket",
   cellar       = "Cellar",
@@ -10,11 +11,11 @@
   gitlab       = "GitLab",
   local        = "Local",
   repository   = "Repository",
-  standard     = "CRAN",
+  standard     = "Repository",
   url          = "URL",
   xgit         = "Git"
 )
 
-renv_alias <- function(text) {
-  `_renv_aliases`[[text]] %||% text
+alias <- function(text) {
+  the$aliases[[text]] %||% text
 }
