@@ -12,6 +12,24 @@ renv_cache_version_previous <- function() {
   paste("v", number - 1L, sep = "")
 }
 
+renv_cache_init <- function() {
+
+  if (testing() || checking())
+    return()
+
+  root <- renv_paths_root()
+  if (!file.exists(root))
+    return()
+
+  cache <- renv_paths_cache()
+  ensure_directory(cache)
+
+  ignorefile <- file.path(cache, ".renvignore")
+  if (!file.exists(ignorefile))
+    writeLines("*", con = ignorefile)
+
+}
+
 # given a record, find a compatible version of that package in the cache,
 # using a computed hash if available; if no hash is available, then try
 # to match based on the package name + version
@@ -74,6 +92,10 @@ renv_cache_find <- function(record) {
     record <- record[nzchar(record)]
     dcf <- dcf[nzchar(dcf)]
 
+    # drop remote fields for standard remotes
+    if (identical(dcf$RemoteType, "standard"))
+      dcf <- dcf[grep("^Remote(?!s)", names(dcf), invert = TRUE, perl = TRUE)]
+
     # check identical
     lhs <- keep(record, fields)
     rhs <- keep(dcf, fields)
@@ -170,6 +192,10 @@ renv_cache_synchronize_impl <- function(cache, record, linkable, path) {
   if (!writable)
     return(FALSE)
 
+  # obtain lock on the cache
+  lockpath <- file.path(parent, ".cache.lock")
+  renv_scope_lock(lockpath)
+
   # if we already have a cache entry, back it up
   restore <- renv_file_backup(cache)
   defer(restore())
@@ -267,7 +293,7 @@ renv_cache_diagnose_corrupt_metadata <- function(paths, problems, verbose) {
 
     # nocov start
     if (verbose) {
-      renv_pretty_print(
+      caution_bullets(
         "The following package(s) are missing 'Meta/package.rds':",
         renv_cache_format_path(bad),
         "These packages should be purged and reinstalled."
@@ -296,7 +322,7 @@ renv_cache_diagnose_corrupt_metadata <- function(paths, problems, verbose) {
 
     # nocov start
     if (verbose) {
-      renv_pretty_print(
+      caution_bullets(
         "The following package(s) have corrupt 'Meta/package.rds' files:",
         renv_cache_format_path(bad),
         "These packages should be purged and reinstalled."
@@ -327,7 +353,7 @@ renv_cache_diagnose_missing_descriptions <- function(paths, problems, verbose) {
 
   # nocov start
   if (verbose) {
-    renv_pretty_print(
+    caution_bullets(
       "The following packages are missing DESCRIPTION files in the cache:",
       renv_cache_format_path(bad),
       "These packages should be purged and reinstalled."
@@ -361,7 +387,7 @@ renv_cache_diagnose_bad_hash <- function(paths, problems, verbose) {
     fmt <- "%s %s [Hash: %s != %s]"
     entries <- sprintf(fmt, lhs$Package, lhs$Version, lhs$Hash, rhs$Hash)
 
-    renv_pretty_print(
+    caution_bullets(
       "The following packages have incorrect hashes:",
       entries,
       "Consider using `renv::rehash()` to re-hash these packages."
@@ -404,7 +430,7 @@ renv_cache_diagnose_wrong_built_version <- function(paths, problems, verbose) {
     # nocov start
     if (verbose) {
 
-      renv_pretty_print(
+      caution_bullets(
         "The following packages have no 'Built' field recorded in their DESCRIPTION file:",
         paths[isna],
         "renv is unable to validate the version of R this package was built for."
@@ -442,7 +468,7 @@ renv_cache_diagnose_wrong_built_version <- function(paths, problems, verbose) {
   # nocov start
   if (verbose) {
 
-    renv_pretty_print(
+    caution_bullets(
       "The following packages in the cache were built for a different version of R:",
       renv_cache_format_path(paths[wrong]),
       "These packages will need to be purged and reinstalled."
@@ -525,37 +551,8 @@ renv_cache_format_path <- function(paths) {
 # nocov end
 
 renv_cache_clean_empty <- function(cache = NULL) {
-
-  # no-op for Solaris
-  if (renv_platform_solaris())
-    return(FALSE)
-
-  # move to cache root
   caches <- cache %||% renv_paths_cache()
-  for (cache in caches)
-    renv_cache_clean_empty_impl(cache)
-
-  TRUE
-
-}
-
-renv_cache_clean_empty_impl <- function(cache) {
-
-  # move to cache directory
-  renv_scope_wd(cache)
-
-  # construct system command for removing empty directories
-  action <- "removing empty directories"
-  if (renv_platform_windows()) {
-    args <- c(".", ".", "/S", "/MOVE")
-    renv_system_exec("robocopy", args, action, 0:8)
-  } else {
-    args <- c(".", "-type", "d", "-empty", "-delete")
-    renv_system_exec("find", args, action)
-  }
-
-  TRUE
-
+  map(caches, renv_cleanse_empty)
 }
 
 renv_cache_package_validate <- function(path) {
