@@ -42,6 +42,7 @@ the$install_step_width <- 48L
 #' fulfilling the installation request.
 #'
 #' @inherit renv-params
+#'
 #' @param packages Either `NULL` (the default) to install all packages required
 #'  by the project, or a character vector of packages to install. renv
 #'  supports a subset of the remotes syntax used for package installation,
@@ -60,6 +61,18 @@ the$install_step_width <- 48L
 #'  So to install from the `subdir` subdirectory of GitHub package
 #'  `username/repo` you'd use `"username/repo:subdir`.
 #'
+#' @param exclude Packages which should not be installed. `exclude` is useful
+#'   when using `renv::install()` to install all dependencies in a project,
+#'   except for a specific set of packages.
+#'
+#' @param verbose Boolean; report output from `R CMD build` and `R CMD INSTALL`
+#'   during installation? When `NULL` (the default), the value of `config$install.verbose()`
+#'   will be used. When `FALSE`, installation output will be emitted only if
+#'   a package fails to install.
+#'
+#' @param lock Boolean; update the `renv.lock` lockfile after the successful
+#'   installation of the requested packages?
+#'
 #' @return A named list of package records which were installed by renv.
 #'
 #' @export
@@ -92,12 +105,15 @@ the$install_step_width <- 48L
 #' }
 install <- function(packages = NULL,
                     ...,
+                    exclude      = NULL,
                     library      = NULL,
                     type         = NULL,
                     rebuild      = FALSE,
                     repos        = NULL,
                     prompt       = interactive(),
                     dependencies = NULL,
+                    verbose      = NULL,
+                    lock         = FALSE,
                     project      = NULL)
 {
   renv_consent_check()
@@ -120,6 +136,10 @@ install <- function(packages = NULL,
     renv_scope_binding(the, "install_dependency_fields", fields)
   }
 
+  # handle 'verbose'
+  verbose <- verbose %||% config$install.verbose()
+  renv_scope_options(renv.config.install.verbose = verbose)
+
   # set up library paths
   libpaths <- renv_libpaths_resolve(library)
   renv_scope_libpaths(libpaths)
@@ -150,6 +170,11 @@ install <- function(packages = NULL,
 
   # figure out which packages we should install
   packages <- names(remotes) %||% renv_snapshot_dependencies(project, dev = TRUE)
+
+  # apply exclude parameter
+  if (length(exclude))
+    packages <- setdiff(packages, exclude)
+
   if (empty(packages)) {
     writef("- There are no packages to install.")
     return(invisible(list()))
@@ -213,6 +238,14 @@ install <- function(packages = NULL,
   # check loaded packages and inform user if out-of-sync
   renv_install_postamble(names(records))
 
+  # update lockfile if requested
+  if (lock) {
+    renv_snapshot_auto_suppress_next()
+    lockfile <- renv_lockfile_load(project = project)
+    lockfile <- renv_lockfile_modify(lockfile, records)
+    renv_lockfile_save(lockfile, project = project)
+  }
+
   invisible(records)
 }
 
@@ -395,8 +428,9 @@ renv_install_package <- function(record) {
     feedback <- paste0(feedback, " and cached")
   }
 
+  verbose <- config$install.verbose()
   elapsed <- difftime(after, before, units = "auto")
-  renv_install_step_ok(feedback, elapsed = elapsed)
+  renv_install_step_ok(feedback, elapsed = elapsed, verbose = verbose)
 
   invisible()
 
@@ -427,7 +461,7 @@ renv_install_package_cache <- function(record, cache, linker) {
   defer(callback())
 
   # report successful link to user
-  renv_install_step_start("Installing", record$Package)
+  renv_install_step_start("Installing", record$Package, verbose = FALSE)
 
   before <- Sys.time()
   linker(cache, target)
@@ -502,7 +536,8 @@ renv_install_package_impl_prebuild <- function(record, path, quiet) {
     return(path)
   }
 
-  renv_install_step_start("Building", record$Package)
+  verbose <- config$install.verbose()
+  renv_install_step_start("Building", record$Package, verbose = verbose)
 
   before <- Sys.time()
   package <- record$Package
@@ -540,7 +575,10 @@ renv_install_package_impl <- function(record, quiet = TRUE) {
 
   # check whether we should build before install
   path <- renv_install_package_impl_prebuild(record, path, quiet)
-  renv_install_step_start("Installing", record$Package)
+
+  # report start of installation to user
+  verbose <- config$install.verbose()
+  renv_install_step_start("Installing", record$Package, verbose = verbose)
 
   # run user-defined hooks before, after install
   options <- renv_install_package_options(package)
@@ -635,13 +673,13 @@ renv_install_test <- function(package) {
   # we use 'loadNamespace()' rather than 'library()' because some packages might
   # intentionally throw an error in their .onAttach() hooks
   # https://github.com/rstudio/renv/issues/1611
-  code <- substitute({
+  code <- inject({
     options(warn = 1L)
     loadNamespace(package)
-  }, list(package = package))
+  })
 
   # write it to a tempfile
-  script <- renv_scope_tempfile("renv-install-")
+  script <- renv_scope_tempfile("renv-install-", fileext = ".R")
   writeLines(deparse(code), con = script)
 
   # check that the package can be loaded in a separate process
@@ -784,6 +822,7 @@ renv_install_preflight_permissions <- function(library) {
 renv_install_preflight <- function(project, libpaths, records) {
 
   library <- nth(libpaths, 1L)
+  records <- filter(records, Negate(is.function))
 
   all(
     renv_install_preflight_unknown_source(records),
@@ -800,14 +839,20 @@ renv_install_report <- function(records, library) {
   )
 }
 
-renv_install_step_start <- function(action, package) {
+renv_install_step_start <- function(action, package, verbose = FALSE) {
+
+  if (verbose)
+    return(writef("- %s %s ...", action, package))
+
   message <- sprintf("- %s %s ... ", action, package)
   printf(format(message, width = the$install_step_width))
+
 }
 
-renv_install_step_ok <- function(..., elapsed = NULL) {
+renv_install_step_ok <- function(..., elapsed = NULL, verbose = FALSE) {
   renv_report_ok(
     message = paste(..., collapse = ""),
-    elapsed = elapsed
+    elapsed = elapsed,
+    verbose = verbose
   )
 }
