@@ -490,7 +490,6 @@ test_that("custom dependency fields in install are supported", {
 
   install("breakfast", dependencies = c("strong", "Config/Needs/protein"))
   expect_true(renv_package_installed("egg"))
-
 })
 
 test_that("install has user-friendly output", {
@@ -597,3 +596,130 @@ test_that("install() reports failure when a 'bad' binary is installed", {
   remove("bread")
 
 })
+
+test_that("install() respects dependencies argument", {
+  skip_on_cran()
+  project <- renv_tests_scope()
+  init()
+
+  contents <- heredoc("
+    Type: Project
+    Depends: coffee
+    Imports: bread
+    Suggests: muffin
+  ")
+
+  writeLines(contents, con = "DESCRIPTION")
+  install(dependencies = "Imports")
+
+  expect_true(renv_package_installed("bread"))
+  expect_false(renv_package_installed("coffee"))
+  expect_false(renv_package_installed("muffin"))
+})
+
+test_that("install() succeeds even some repositories cannot be queried", {
+  renv_tests_scope()
+
+  repos <- getOption("repos")
+  repos[["NARC"]] <- file.path(repos[["CRAN"]], "missing")
+  renv_scope_options(repos = repos)
+
+  init()
+  install("bread")
+  expect_true(renv_package_installed("bread"))
+})
+
+test_that("install() doesn't duplicate authentication headers", {
+  renv_scope_envvars(RENV_DOWNLOAD_METHOD = "libcurl")
+  project <- renv_tests_scope()
+  init()
+  install("kevinushey/skeleton")
+  expect_true(renv_package_installed("skeleton"))
+})
+
+test_that("install() stores repository information for installed packages", {
+
+  project <- renv_tests_scope(isolated = TRUE)
+  init()
+
+  # unset repository option
+  repos <- getOption("repos")
+  renv_scope_options(repos = character())
+
+  # try to install a package
+  writeLines("library(bread)", con = "_deps.R")
+  install("bread", repos = c(TEST = unname(repos)))
+
+  # create a lockfile
+  snapshot()
+
+  # validate that the repository information is stored
+  lockfile <- renv_lockfile_read("renv.lock")
+  record <- lockfile$Packages$bread
+  expect_equal(!!record$Source, "Repository")
+  expect_equal(!!record$Repository, !!unname(repos))
+
+  # now, add the repository back; it should then be aliased in lockfile
+  options(repos = c(TEST = unname(repos)))
+  snapshot()
+
+  lockfile <- renv_lockfile_read("renv.lock")
+  record <- lockfile$Packages$bread
+  expect_equal(!!record$Source, "Repository")
+  expect_equal(!!record$Repository, "TEST")
+
+})
+
+test_that("install() lazily resolves project remotes", {
+
+  project <- renv_tests_scope()
+  init()
+
+  writeLines("Remotes: kevinushey/skeleton", con = "DESCRIPTION")
+  install("bread")
+  expect_false(renv_package_installed("skeleton"))
+
+})
+
+test_that("install() records the repository used to retrieve a package", {
+
+  project <- renv_tests_scope(isolated = TRUE)
+  init()
+
+  url <- unname(getOption("repos"))
+  local({
+    renv_scope_options(repos = character())
+    install("bread", repos = c(TEST = url), rebuild = TRUE)
+  })
+
+  dcf <- renv_description_read(package = "bread")
+  expect_equal(!!dcf$RemoteRepos, !!url)
+  expect_equal(!!dcf$RemoteReposName, "TEST")
+
+})
+
+test_that("recursive dependency versions are properly resolved", {
+
+  project <- renv_tests_scope()
+  init()
+
+  install("phone@0.1.0")
+  install("jamie")
+
+  expect_equal(renv_package_version("phone"), "1.0.0")
+
+})
+
+test_that("install(lock = TRUE) updates lockfile", {
+
+  project <- renv_tests_scope()
+  init()
+
+  install("breakfast", lock = TRUE)
+  lockfile <- renv_lockfile_load(project = project)
+  records <- renv_lockfile_records(lockfile)
+
+  expect_true("bread" %in% names(records))
+  expect_true("breakfast" %in% names(records))
+
+})
